"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const background_image_1 = __importDefault(require("./src/attribute/background-image"));
const link_1 = __importDefault(require("./src/attribute/link"));
const digit_1 = __importDefault(require("./src/attribute/digit"));
const size_1 = __importDefault(require("./src/attribute/size"));
const text_1 = __importDefault(require("./src/attribute/text"));
const puppeteer_1 = __importDefault(require("./src/fetcher/puppeteer"));
const html_1 = __importDefault(require("./src/provider/html"));
const script_tag_1 = __importDefault(require("./src/provider/script-tag"));
class Scraper {
    constructor(url, contract, attributes = {}) {
        this.defaultAttributes = {
            'background-image': background_image_1.default,
            link: link_1.default,
            digit: digit_1.default,
            size: size_1.default,
            text: text_1.default,
        };
        this.url = url;
        this.contract = contract;
        this.attributes = attributes;
    }
    scrapePage() {
        if (!this.contractIsValid()) {
            throw Error('Your contract is invalid, please check the specifications');
        }
        if (!this.urlIsValid()) {
            throw Error(`The URL "${this.url}" you have provided is invalid`);
        }
        const attributes = this.getAttributes();
        const fetcher = this.getFetcher();
        return fetcher.getPage().then((page) => {
            return this.getScrapedItems(page, attributes);
        });
    }
    getScrapedItems(page, attributes) {
        return this.getProvider(page, attributes).getScrapedItems();
    }
    getAttributes() {
        return Object.assign(this.defaultAttributes, this.attributes);
    }
    urlIsValid() {
        try {
            new URL(this.url);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    contractIsValid() {
        return !(this.contract === null || this.contract === undefined);
    }
    getFetcher() {
        return new puppeteer_1.default(this.url);
    }
    getProvider(page, attributes) {
        if (this.contract.scriptTagSelector) {
            return new script_tag_1.default(page, this.contract, attributes);
        }
        return new html_1.default(page, this.contract, attributes);
    }
}
exports.default = Scraper;
