"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_promise_1 = __importDefault(require("request-promise"));
const random_useragent_1 = __importDefault(require("random-useragent"));
const encoding_1 = require("../tools/encoding");
class RequestFetcher {
    constructor(url) {
        this.url = url;
    }
    getPage() {
        return this.getPageResponse().then((response) => {
            const { headers, body } = response;
            const encoding = encoding_1.guessEncoding(headers['Content-Type'], body);
            const page = {
                encoding,
                contents: encoding_1.encodePageContents(encoding, body),
                url: this.url,
            };
            return page;
        });
    }
    getRequestLibrary() {
        return request_promise_1.default;
    }
    getPageResponse() {
        const userAgent = random_useragent_1.default.getRandom();
        return this.getRequestLibrary().get({
            url: this.url,
            gzip: true,
            proxy: null,
            encoding: 'utf-8',
            resolveWithFullResponse: true,
            headers: {
                'Content-Type': 'text/html; charset=utf-8',
                'Accept': 'text/html',
                'Accept-Language': 'fr-FR',
                'User-Agent': userAgent,
            }
        });
    }
}
exports.default = RequestFetcher;
