"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cheerio = __importStar(require("cheerio"));
const cheerio_eq_1 = __importDefault(require("cheerio-eq"));
const lodash_get_1 = __importDefault(require("lodash.get"));
class ScriptTagProvider {
    constructor(page, contract, attributes) {
        this.page = page;
        this.contract = contract;
        this.attributes = attributes;
        this.$ = cheerio.load(this.page.contents);
        this.contents = this.parseScriptTagContents();
    }
    mapElementToProperty(item, options) {
        const { type, selector } = options;
        const value = lodash_get_1.default(item, selector);
        const AttributeType = this.attributes[type];
        if (AttributeType === undefined) {
            throw Error(`The attribute type ${type} isn't defined, did you pass it to the scraper?`);
        }
        const scrapedAttribute = new AttributeType(value, this.page.url);
        return scrapedAttribute.value;
    }
    parseScriptTagContents() {
        const { scriptTagSelector } = this.contract;
        const contents = cheerio_eq_1.default(this.$, scriptTagSelector).html();
        if (!contents || contents.length === 0)
            return;
        return JSON.parse(contents);
    }
    getScrapedItems() {
        const scrapedItems = [];
        let items = lodash_get_1.default(this.contents, this.contract.itemSelector) || this.contents;
        if (!Array.isArray(items)) {
            items = [items];
        }
        items.forEach((item) => {
            const scrapedItem = {};
            Object.entries(this.contract.attributes).forEach(([name, options]) => {
                if (options.itemSelector !== undefined) {
                    const childElements = lodash_get_1.default(item, options.itemSelector) || [];
                    scrapedItem[name] = [];
                    childElements.forEach((childElement) => {
                        const childValues = {};
                        Object.entries(options.attributes).forEach(([childName, childOptions]) => {
                            childValues[childName] = this.mapElementToProperty(childElement, childOptions);
                        });
                        scrapedItem[name].push(childValues);
                    });
                }
                else {
                    scrapedItem[name] = this.mapElementToProperty(item, options);
                }
            });
            scrapedItems.push(scrapedItem);
        });
        return scrapedItems;
    }
}
exports.default = ScriptTagProvider;
